==> create-mashup-app-2.0.2.tgz: package/src/config/rollup.config.default.js <==
const path = require('path');
const {red, green, cyan} = require('chalk');
const {identity} = require('lodash');
const cosmiconfig = require('cosmiconfig');
const errors = require('./errors.json');
const babel = require('rollup-plugin-babel');
const postcss = require('rollup-plugin-postcss');
const json = require('rollup-plugin-json');
const commonjs = require('rollup-plugin-commonjs');
const resolve = require('rollup-plugin-node-resolve');
const iife = require('rollup-plugin-iife');
const {terser} = require('rollup-plugin-terser');
const typescript = require('rollup-plugin-typescript');
const mashup = require('../plugins/mashupRollupPlugin');
const chunkInfo = require('../plugins/chunkInfoPlugin');
const cleanOutput = require('../plugins/cleanPlugin');

async function getPostcssConfig(dir, minify) {
  const postcssConfigSearchResult = await cosmiconfig('postcss').search(dir);
  const postcssConfig = (postcssConfigSearchResult &&
    postcssConfigSearchResult.config) || {
    plugins: [
      require('postcss-import'),
      require('postcss-global-import'),
      require('postcss-each'),
      require('postcss-mixins'),
      require('postcss-simple-vars'),
      require('postcss-nested-ancestors'),
      require('postcss-nested'),
      require('postcss-current-selector'),
      require('postcss-color-function'),
      require('postcss-calc'),
      require('postcss-inline-svg'),
      require('autoprefixer')
    ]
  };
  return postcssConfig;
}

module.exports = async ({
  dir = process.cwd(),
  minify,
  dependencyReplace = true,
  target,
  outputDir,
  basePath
}) => {
  const configSearchResult = await cosmiconfig('mashup').search(dir);
  if (!configSearchResult) {
    throw new Error(errors.configNotFound);
  }

  const {config: mashupConfig, filepath} = configSearchResult;

  const entryPoint = path.resolve(dir, mashupConfig.entry || 'src/index.js');
  const targetPackageJson = require(path.join(dir, 'package.json'));
  let configTransform = mashupConfig.configTransform;

  console.log(` - loaded mashup config from ${cyan(filepath)}`);
  console.log(
    ` - minification is ${minify ? green('enabled') : red('disabled')}`
  );
  console.log(` - entry point: ${cyan(entryPoint)}`);
  if (typeof configTransform === 'function') {
    console.log(
      ' - using custom config transform (function declared in config)'
    );
  } else if (typeof configTransform === 'string') {
    const configTransformFilePath = path.join(dir, configTransform);
    console.log(
      ` - using custom config transform from ${cyan(configTransformFilePath)}`
    );
    configTransform = require(configTransformFilePath);
  }
  if (typeof configTransform !== 'function') {
    configTransform = identity;
  }

  const mashupName = mashupConfig.name || targetPackageJson.name;
  const mashupVersion = targetPackageJson.version;
  const mashupModuleName = mashupConfig.moduleName || mashupName;
  const {dependencies = []} = mashupConfig;

  // collect all dependencies that should be excluded from the bundle and
  // instead taken from global object
  const externalDependencies = dependencies.map(
    dependency =>
      dependency.provideModule || dependency.moduleName || dependency
  );
  // all external dependencies are dumped into one object on window, and can be taken from there
  // we also need unique name for this particular external, so we generate it with
  // a bit of randomness on each build
  const randomString = Math.random()
    .toString(36)
    .substring(2, 7);
  const mashupGlobalName = `__${mashupName
    .split('-')
    .join('')}${mashupVersion.split('.').join('')}_${randomString}`;
  const mashupDependenciesGlobalName = `${mashupGlobalName}_deps`;
  const mashupChunksGlobalName = `${mashupGlobalName}_chunks`;
  const mashupImportGlobalName = `${mashupGlobalName}_import`;
  outputDir = outputDir ? path.resolve(outputDir) : path.join(dir, 'dist');

  const rollupConfig = {
    input: [entryPoint],
    output: {
      dir: outputDir,
      format: 'es',
      name: mashupGlobalName,
      sourcemap: true,
      globals: localDependencyName => {
        if (localDependencyName.startsWith(outputDir)) {
          const chunkName = localDependencyName
            .replace(outputDir, '')
            .substring(1);
          // all code-split chunks are written as keys of <something>_chunks object
          return `window.${mashupChunksGlobalName}['./${chunkName}']`;
        }
        // all other modules are externals (i.e. react) and are extracted from
        // keys of <something>_deps
        return `window.${mashupDependenciesGlobalName}['${localDependencyName}']`;
      }
    },
    experimentalCodeSplitting: true,
    external: dependencyReplace ? externalDependencies : null,
    plugins: [
      typescript({
        jsx: 'React',
        include: /\.tsx?/
      }),
      babel({
        runtimeHelpers: true,
        // we need to include @babel/runtime most of the times
        // and it needs transpiling to have proper tree-shaking
        exclude: /node_modules(\/|\\)(?!(@babel))/,
        presets: [
          ['@babel/preset-env', {modules: false}],
          '@babel/preset-react'
        ],
        plugins: [
          '@babel/plugin-transform-runtime',
          '@babel/plugin-syntax-dynamic-import',
          '@babel/plugin-proposal-object-rest-spread',
          '@babel/plugin-proposal-class-properties',
          '@babel/plugin-transform-property-literals',
          '@babel/plugin-transform-member-expression-literals'
        ]
      }),
      commonjs(),
      resolve({
        jsnext: true,
        main: true,
        extensions: ['.js', '.jsx']
      }),
      postcss({
        modules: {
          generateScopedName: className =>
            `${className}__${mashupName}__${mashupVersion.split('.').join('-')}`
        },
        inject: true,
        // babel prioritizes config file (.babelrc) over hardcoded config
        // but postcss doesn't, so we need to manually search for custom config file
        // luckily, it uses cosmiconfig as well
        ...(await getPostcssConfig(dir, minify))
      }),
      json(),
      iife(),
      mashup({
        basePath,
        entryPoint,
        dependencies,
        target,
        mashupModuleName,
        mashupGlobalName,
        mashupChunksGlobalName,
        mashupImportGlobalName,
        mashupDependenciesGlobalName
      }),
      chunkInfo(),
      cleanOutput({inputDir: dir})
    ]
  };

  if (minify) {
    rollupConfig.plugins.push(terser());
  }

  return configTransform(rollupConfig);
};

==> create-mashup-app-2.4.0.tgz: package/src/config/rollup.config.default.js <==
const path = require('path');
const {red, green, cyan} = require('chalk');
const {identity} = require('lodash');
const cosmiconfig = require('cosmiconfig');
const errors = require('./errors.json');
const babel = require('rollup-plugin-babel');
const postcss = require('rollup-plugin-postcss');
const json = require('rollup-plugin-json');
const commonjs = require('rollup-plugin-commonjs');
const resolve = require('rollup-plugin-node-resolve');
const iife = require('rollup-plugin-iife');
const {terser} = require('rollup-plugin-terser');
const typescript2 = require('rollup-plugin-typescript2');
const replace = require('rollup-plugin-replace');
const mashup = require('../plugins/mashupRollupPlugin');
const chunkInfo = require('../plugins/chunkInfoPlugin');
const cleanOutput = require('../plugins/cleanPlugin');
const includePaths = require('rollup-plugin-includepaths');

async function getPostcssConfig(dir, getVariables = () => ({})) {
  const postcssConfigSearchResult = await cosmiconfig('postcss').search(dir);
  const postcssConfig = (postcssConfigSearchResult &&
    postcssConfigSearchResult.config) || {
    plugins: [
      require('postcss-import'),
      require('postcss-global-import'),
      require('postcss-each'),
      require('postcss-mixins'),
      require('postcss-simple-vars')({
        variables: () => {
          const vars = getVariables();
          return vars || {};
        }
      }),
      require('postcss-nested-ancestors'),
      require('postcss-nested'),
      require('postcss-current-selector'),
      require('postcss-color-function'),
      require('postcss-calc'),
      require('postcss-inline-svg'),
      require('autoprefixer')
    ]
  };
  return postcssConfig;
}

module.exports = async ({
  dir = process.cwd(),
  minify,
  dependencyReplace = true,
  target,
  outputDir,
  basePath
}) => {
  const configSearchResult = await cosmiconfig('mashup').search(dir);
  if (!configSearchResult) {
    throw new Error(errors.configNotFound);
  }

  const {config: mashupConfig, filepath} = configSearchResult;

  const entryPoint = path.resolve(dir, mashupConfig.entry || 'src/index.js');
  const targetPackageJson = require(path.join(dir, 'package.json'));
  let configTransform = mashupConfig.configTransform;

  console.log(` - loaded mashup config from ${cyan(filepath)}`);
  console.log(
    ` - minification is ${minify ? green('enabled') : red('disabled')}`
  );
  console.log(` - entry point: ${cyan(entryPoint)}`);
  if (typeof configTransform === 'function') {
    console.log(
      ' - using custom config transform (function declared in config)'
    );
  } else if (typeof configTransform === 'string') {
    const configTransformFilePath = path.join(dir, configTransform);
    console.log(
      ` - using custom config transform from ${cyan(configTransformFilePath)}`
    );
    configTransform = require(configTransformFilePath);
  }
  if (typeof configTransform !== 'function') {
    configTransform = identity;
  }

  const mashupName = mashupConfig.name || targetPackageJson.name;
  const mashupVersion = targetPackageJson.version;
  const mashupModuleName = mashupConfig.moduleName || mashupName;
  const {dependencies = []} = mashupConfig;

  // collect all dependencies that should be excluded from the bundle and
  // instead taken from global object
  const externalDependencies = dependencies.map(
    dependency =>
      dependency.provideModule || dependency.moduleName || dependency
  );
  // all external dependencies are dumped into one object on window, and can be taken from there
  // we also need unique name for this particular external, so we generate it with
  // a bit of randomness on each build
  const randomString = Math.random()
    .toString(36)
    .substring(2, 7);
  const mashupGlobalName = `__${mashupName.replace(
    /[- ]/g,
    ''
  )}${mashupVersion.replace(/[\.\+-]/g, '')}_${randomString}`;
  const mashupDependenciesGlobalName = `${mashupGlobalName}_deps`;
  const mashupChunksGlobalName = `${mashupGlobalName}_chunks`;
  const mashupImportGlobalName = `${mashupGlobalName}_import`;
  outputDir = outputDir ? path.resolve(outputDir) : path.join(dir, 'dist');

  const cssVariablesConfig = mashupConfig.cssVariables;
  const getCssVariables = () => {
    if (typeof cssVariablesConfig === 'string') {
      return require(cssVariablesConfig);
    } else if (typeof cssVariablesConfig === 'function') {
      return cssVariablesConfig();
    }

    return cssVariablesConfig;
  };

  const rollupConfig = {
    input: [entryPoint],
    output: {
      dir: outputDir,
      format: 'es',
      name: mashupGlobalName,
      sourcemap: !minify,
      globals: localDependencyName => {
        if (localDependencyName.startsWith(outputDir)) {
          const chunkName = localDependencyName
            .replace(outputDir, '')
            .substring(1);
          // all code-split chunks are written as keys of <something>_chunks object
          return `window.${mashupChunksGlobalName}['./${chunkName}']`;
        }
        // all other modules are externals (i.e. react) and are extracted from
        // keys of <something>_deps
        return `window.${mashupDependenciesGlobalName}['${localDependencyName}']`;
      }
    },
    experimentalCodeSplitting: true,
    external: dependencyReplace ? externalDependencies : null,
    plugins: [
      resolve({
        jsnext: true,
        module: true,
        extensions: ['.js', '.jsx']
      }),
      // typescript, babel and commonjs can usually perform DCE based
      // on defined replaced strings, so replace goes before them
      replace({
        'process.env.NODE_ENV': JSON.stringify(
          minify ? 'production' : 'development'
        )
      }),
      includePaths({
        paths: [dir],
        extensions: ['.js', '.jsx', '.ts', '.tsx']
      }),
      typescript2({
        jsx: 'React',
        include: /\.tsx?/,
        objectHashIgnoreUnknownHack: true,
        cacheRoot: './node_modules/.rpt2_cache',
        tsconfigDefaults: {
          compilerOptions: {
            allowSyntheticDefaultImports: true,
            module: 'es2015',
            target: 'es6',
            jsx: 'react'
          }
        }
      }),
      json(),
      babel({
        runtimeHelpers: true,
        exclude: /node_modules(\/|\\)(.*)/,
        extensions: ['.js', '.jsx', '.ts', '.tsx', '.json'],
        presets: [
          ['@babel/preset-env', {modules: false, useBuiltIns: 'usage'}],
          '@babel/preset-react'
        ],
        plugins: [
          '@babel/plugin-transform-runtime',
          '@babel/plugin-syntax-dynamic-import',
          '@babel/plugin-proposal-object-rest-spread',
          '@babel/plugin-proposal-class-properties',
          '@babel/plugin-transform-property-literals',
          '@babel/plugin-transform-member-expression-literals'
        ]
      }),
      commonjs({
        namedExports: targetPackageJson.namedExports
      }),
      postcss({
        modules: {
          generateScopedName: className =>
            `${className}__${mashupName}__${mashupVersion
              .split('.')
              .join('-')}`,
          camelCase: true
        },
        inject: true,
        // babel prioritizes config file (.babelrc) over hardcoded config
        // but postcss doesn't, so we need to manually search for custom config file
        // luckily, it uses cosmiconfig as well
        ...(await getPostcssConfig(dir, getCssVariables))
      }),
      iife(),
      mashup({
        basePath,
        entryPoint,
        dependencies,
        target,
        mashupModuleName,
        mashupGlobalName,
        mashupChunksGlobalName,
        mashupImportGlobalName,
        mashupDependenciesGlobalName
      }),
      chunkInfo(),
      cleanOutput({inputDir: dir})
    ]
  };

  if (minify) {
    rollupConfig.plugins.push(terser());
  }

  return configTransform(rollupConfig);
};
